use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Aas
Adam
Alders
Alex
Alexandr
Alexey
Andreas
Anwar
Bill
Book
Bron
Burke
Chase
Christopher
Chrysostomos
Ciornii
Couzins
DAVIDRW
Dan
Daniel
David
Denaxas
Etheridge
FWILES
Father
Finch
Froehlich
Gavin
Gisle
Golden
Gondwana
Graeme
Grossmann
Hanak
Hans
Hay
Hedlund
Hukins
Ian
JJ
Jacob
Jakub
Kaji
Kapranoff
Karaban
Karen
Kennedy
Kilgore
Koenig
LWP
Lapworth
Leo
Lipcon
Madsen
Mann
Mark
Merelo
Michael
Mike
Mohammad
Olaf
Ondrej
Peter
Peters
Protocol
Rabbitson
Randy
Rezic
Robert
Rolf
Schilli
Schwern
Sean
Shoichi
Sjogren
Skyttä
Slaven
Spiros
Stauner
Steffen
Steffen_Ullrich
Steve
SteveHay
Stone
Stosberg
Thompson
Tim
Todd
Tom
Tony
Toru
Tourbin
Ullrich
Ville
Wheeler
Whitener
Wilk
Yamaguchi
Yuri
Yury
Zavarin
Zefram
adamk
alexchorny
amir
amire80
andreas
asjo
at
brong
capoeirab
cjm
cpansprout
dagolden
david
davidrw
denaxas
dot
drieux
ether
gisle
github
gpeters
grinnz
hfroehlich
https
iank
jefflee
jjmerelo
john9art
jwilk
ka
leo
lib
mark
mohammad
murphy
olaf
ondrej
phrstbrn
randy
rg
ribasushi
ruff
sasao
sburke
schwern
shaohua
skaji
slaven
sprout
talby
tech
tim
todd
tom
uid39246
ville
waif
wfmann
yury
zefram
zigorou
